{{ persona }}

{{ tools_prompt }}

Conversation History:
{{ conversation_history }}

{% if has_tools %}
Assistant: The tool has executed successfully. Based on the tool output above:
1. If you need to use another tool, respond with the tool JSON.
2. Otherwise, provide your final response that INCLUDES the actual tool results/data.
3. Do NOT just describe what you did - show the actual results. Unless mentioned otherwise.
DO NOT output JSON unless calling a tool.
{% else %}
Assistant: Based on the information above, provide your response in natural language.
{% endif %}
